//
// JTypist v3.0 - Java typing tutor program for UNIX systems
// Copyright (C) 1998  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// ControlGUI class - links lesson data and other frames together
//
import java.awt.*;
import java.applet.*;

public class ControlGUI extends Frame {
	private boolean inAnApplet = true;

	// other frames that we have to control
	private TutorialGUI tutorial = null;
	private HelpGUI help = null;
	private AboutGUI about = null;
	private AAboutGUI aabout = null;
	private ExerciseGUI exercise = null;
	private StatisticsGUI statistics = null;

	// lesson library data
	private LessonSeriesLibrary lesson_library = null;

	// menubar GUI components
	private MenuBar menu_bar;
	private Menu file_menu, window_menu, help_menu;
	private MenuItem file_exit;
	private MenuItem window_help, window_tutorial, window_exercise,
			window_statistics;
	private MenuItem help_about, help_on_typist;

	// GUI components
	private Button prev_button, next_button;
	private Choice series_choice;
	private Label series_label;
	private List lesson_list;


	// constructor initializes the whole GUI set
	public ControlGUI( boolean inAnApplet,
			LessonSeriesLibrary lesson_library ) {
		this.inAnApplet = inAnApplet;

		// save our assigned lesson library
		this.lesson_library = lesson_library;

		// create GUI components
		setTitle( "Typist v3.0 Control Window" );
		setLayout( new BorderLayout() );

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu( "File" );
		menu_bar.add( file_menu );
		file_exit = new MenuItem( "Exit" );
		file_menu.add( file_exit );
		// Window menu - do everything except add the menu items
		// to the bar; this we do in the routines that register
		// each sub-GUI with us
		window_menu = new Menu( "Window" );
		menu_bar.add( window_menu );
		window_help = new MenuItem( "Help" );
		//window_menu.add( window_help );
		window_tutorial = new MenuItem( "Tutorial" );
		//window_menu.add( window_tutorial );
		window_exercise = new MenuItem( "Exercise" );
		//window_menu.add( window_exercise );
		window_statistics = new MenuItem( "Keyboard Statistics" );
		//window_menu.add( window_statistics );
		// Help menu
		help_menu = new Menu( "Help" );
		menu_bar.add( help_menu );
		menu_bar.setHelpMenu( help_menu );
		help_about = new MenuItem( "About Typist" );
		//help_menu.add( help_about );
		help_on_typist = new MenuItem( "On Typist" );
		//help_menu.add( help_on_typist );

		// create a toolbar panel
		Panel toolbar = new Panel();
		toolbar.setLayout( new BorderLayout() );
		add( "North", toolbar );

		Panel controls = new Panel();
		controls.setLayout( new FlowLayout());
		toolbar.add( "West", controls );
		prev_button = new Button( "<Prev" );
		controls.add( prev_button );
		next_button = new Button( "Next>" );
		controls.add( next_button );
		series_choice = new Choice();
		controls.add( series_choice );

		series_label = new Label();
		toolbar.add( "Center", series_label );

		lesson_list = new List( 10, false );
		lesson_list.setBackground( Color.white );
		add( "Center", lesson_list );

		// create the lesson series choice item, and start
		// with the first series we saw
		for ( int i = 0; i < lesson_library.get_num_series(); i++ ) {
			series_choice.addItem( lesson_library.get_series( i ));
		}
		series_choice.select( 0 );
		handle_series_selection();
	}

	// update the lessons list based on the currently selected
	// series
	private void handle_series_selection() {

		// reset the lesson current set and label description
		lesson_library.set_series(
				series_choice.getSelectedItem() );
		series_label.setText( lesson_library.get_descr() );

		// update the lesson list
		if ( lesson_list.countItems() > 0 )
			lesson_list.clear();
		Lesson lessons[] = lesson_library.get_lessons();
		for ( int i = 0; i < lessons.length; i++ ) {
			if ( lessons[ i ].descr.equals( "" ))
				lesson_list.addItem( lessons[ i ].title );
			else
				lesson_list.addItem( lessons[ i ].title + " - "
						+ lessons[ i ].descr );
		}
		garbage_collect();
	}


	// handle the selection of a lesson list item
	private void handle_lesson_selection() {

		// if something is selected action it
		int selected = lesson_list.getSelectedIndex();
		if ( selected >= 0 ) {

			// if selected is at either end of the list,
			// disable the prev/next buttons
			if ( selected == 0 )
				prev_button.disable();
			else
				prev_button.enable();
			if ( selected == lesson_list.countItems() - 1 )
				next_button.disable();
			else
				next_button.enable();

			// For some reason the List getRows() won't give
			// the right answer about the number of visible
			// rows ???
			// If it did, here would would be the place to
			// jiggle the list to keep the selected item in view

			// now just run the lesson based on lesson type
			Lesson lesson = lesson_library.get_lessons()[ selected ];
			switch ( lesson.type ) {
			case Lesson.TUTORIAL:
				if ( tutorial != null )
					tutorial.show( lesson ); break;
			case Lesson.DRILL:
				if ( exercise != null )
					exercise.show( lesson ); break;
			case Lesson.PARAGRAPH:
				if ( exercise != null )
					exercise.show( lesson ); break;
			}
			garbage_collect();
		}
	}

	// select the previous lesson from the list if available
	private void handle_prev_button() {

		// move to previous list item if available
		int selected = lesson_list.getSelectedIndex();
		if ( selected > 0 ) {
			selected--;
			lesson_list.select( selected );
			handle_lesson_selection();
		}
	}


	// select the next lesson from the list if available
	private void handle_next_button() {

		// move to next list item if available
		int selected = lesson_list.getSelectedIndex();
		if ( selected < lesson_list.countItems() - 1 ) {
			selected++;
			lesson_list.select( selected );
			handle_lesson_selection();
		}
	}


	// handle GUI component actions and events
	public boolean action( Event evt, Object arg ) {
		if ( evt.target instanceof Choice ) {
			handle_series_selection();
		}
		else if ( evt.target instanceof Button ) {
			if ( evt.target == prev_button )
				handle_prev_button();
			else if ( evt.target == next_button )
				handle_next_button();
		}
		else if ( evt.target instanceof MenuItem ) {
			if ( evt.target == file_exit ) {
				if ( inAnApplet ) {
					if ( tutorial != null ) tutorial.dispose();
					if ( exercise != null ) exercise.dispose();
					if ( help != null ) help.dispose();
					if ( statistics != null ) statistics.dispose();
					if ( about != null ) about.dispose();
					if ( aabout != null ) aabout.dispose();
					dispose();
				}
				else
					System.exit( 0 );
			} else if ( evt.target == window_tutorial ) {
				if ( tutorial != null ) tutorial.show();
			} else if ( evt.target == window_exercise ) {
				if ( exercise != null ) exercise.show();
			} else if ( evt.target == window_help ) {
				if ( help != null ) help.show();
			} else if ( evt.target == window_statistics ) {
				if ( statistics != null ) statistics.show();
			} else if ( evt.target == help_about ) {
				if ( inAnApplet ) {
					if ( aabout != null ) aabout.show();
				} else {
					if ( about != null ) about.show();
			} } else if ( evt.target == help_on_typist ) {
				if ( help != null ) help.show();
			}
		}
		return ( false );
	}
	public boolean handleEvent( Event evt ) {
		if ( evt.id == Event.WINDOW_DESTROY ) {
			if ( inAnApplet ) {
				if ( tutorial != null ) tutorial.dispose();
				if ( exercise != null ) exercise.dispose();
				if ( help != null ) help.dispose();
				if ( statistics != null ) statistics.dispose();
				if ( about != null ) about.dispose();
				if ( aabout != null ) aabout.dispose();
				dispose();
			}
			else
				System.exit( 0 );
		}
		else if ( evt.target instanceof List ) {
			if ( evt.id == Event.LIST_SELECT )
				handle_lesson_selection();
			else if ( evt.id == Event.LIST_DESELECT ) {
				// just reselect it, and rerun it!
				lesson_list.select(
						((Integer)evt.arg).intValue());
				handle_lesson_selection();
			}
		}
		return ( super.handleEvent( evt ));
	}


	// gather up any garbage that might have accumulated
	private void garbage_collect() { Runtime.getRuntime().gc(); }


	// register the other frames we need to control
	public void register_tutorial( TutorialGUI tutorial ) {
		this.tutorial = tutorial; garbage_collect();
		window_menu.add( window_tutorial );
	}
	public void register_exercise( ExerciseGUI exercise ) {
		this.exercise = exercise; garbage_collect();
		window_menu.add( window_exercise );
	}
	public void register_help( HelpGUI help ) {
		this.help = help; garbage_collect();
		window_menu.add( window_help );
		help_menu.add( help_on_typist );
	}
	public void register_about( AboutGUI about ) {
		if ( ! inAnApplet ) {
			this.about = about; garbage_collect();
			help_menu.add( help_about );
		}
	}
	public void register_aabout( AAboutGUI aabout ) {
		if ( inAnApplet ) {
			this.aabout = aabout; garbage_collect();
			help_menu.add( help_about );
		}
	}
	public void register_statistics( StatisticsGUI statistics ) {
		this.statistics = statistics; garbage_collect();
		window_menu.add( window_statistics );
	}

	// test code
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		ControlGUI control = new ControlGUI( false, new LessonSeriesLibrary() );

		TutorialGUI tutorial = new TutorialGUI( false );
		tutorial.pack(); tutorial.hide();

		KBStatistics kbstats = new KBStatistics( new KBLayout( "US" ));
		StatisticsGUI statistics = new StatisticsGUI( false, kbstats );
		statistics.pack(); statistics.hide();

		ExerciseGUI exercise = new ExerciseGUI( false, kbstats );
		exercise.pack(); exercise.hide();

		HelpGUI help = new HelpGUI( false );
		help.pack(); help.hide();

		AboutGUI about = new AboutGUI( false, control, false );
		about.pack(); about.hide();

		exercise.register_statistics( statistics );
		control.register_about( about );
		control.register_tutorial( tutorial );
		control.register_exercise( exercise );
		control.register_statistics( statistics );
		control.register_help( help );
		control.pack(); control.show();
	}
}
