//
// JTypist v3.0 - Java typing tutor program for UNIX systems
// Copyright (C) 1998  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// ExerciseGUI class - creates a frame to run through a lesson
//
import java.awt.*;
import java.applet.*;

public class ExerciseGUI extends Frame {
	private boolean inAnApplet = true;

	// statistics GUI we could refresh after a lesson
	private StatisticsGUI statistics = null;

	// menubar GUI components
	private MenuBar menu_bar;
	private Menu file_menu;
	private MenuItem file_restart, file_close;

	// GUI components
	private Label title_label;
	private Label descr_label;
	private TextArea reference_text_area;
	private TextArea entry_text_area;
	private TextArea paragraph_text_area;
	// extra stuff to do the card layout
	private Panel text_panel;
	private CardLayout card_layout;


	// symbols used to show bad keys struck, and max timer for
	// keypresses timings
	private final String BADKEY = "*";
	private final String BADCR = "<";
	private final int MAX_DELTA = 2000;

	// data to track progress through the data
	private Lesson lesson = null;
	private KeyStatistics key_stats = new KeyStatistics();
	private long keypress_timestamp = 0;
	private int progress = 0;
	private int cursor = 0;

	// statistics set we are working with, for reference
	KBStatistics stats_set = null;

	// dialogs to show the results of the exercise in
	ExerciseGUIStats finish_dialog;
	AExerciseGUIStats applet_finish_dialog;


	// constructor initializes the whole GUI set
	public ExerciseGUI( boolean inAnApplet, KBStatistics stats_set ) {
		this.inAnApplet = inAnApplet;

		// create GUI components
		setTitle( "Typist v3.0 Exercise Window" );
		setLayout( new BorderLayout() );

		// note the stats set for reference
		this.stats_set = stats_set;

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu( "File" );
		menu_bar.add( file_menu );
		file_restart = new MenuItem( "Restart Lesson" );
		file_menu.add( file_restart );
		file_close = new MenuItem( "Close Window" );
		file_menu.add( file_close );

		// create the labels and text displays
		Panel label_panel = new Panel();
		label_panel.setLayout( new GridLayout( 0, 1 ));
		title_label = new Label();
		label_panel.add( title_label );
		descr_label = new Label();
		label_panel.add( descr_label );
		add( "North", label_panel );

		text_panel = new Panel();
		card_layout = new CardLayout();
		text_panel.setLayout( card_layout );

		Panel drill_panel = new Panel();
		drill_panel.setBackground( Color.white );
		drill_panel.setLayout( new GridLayout( 0, 1 ));
		reference_text_area = new TextArea( 6, 80 );
		reference_text_area.setEditable( false );
		drill_panel.add( reference_text_area );
		entry_text_area = new TextArea( 6, 80 );
		entry_text_area.setEditable( false );
		drill_panel.add( entry_text_area );

		Panel paragraph_panel = new Panel();
		paragraph_panel.setBackground( Color.white );
		paragraph_panel.setLayout( new GridLayout( 0, 1 ));
		paragraph_text_area = new TextArea( 12, 80 );
		paragraph_text_area.setEditable( false );
		paragraph_panel.add( paragraph_text_area );

		text_panel.add( Integer.toString( Lesson.DRILL ),
				drill_panel );
		text_panel.add( Integer.toString( Lesson.PARAGRAPH ),
				paragraph_panel );
		card_layout.show( text_panel, Integer.toString( Lesson.DRILL ));
		add( "Center", text_panel );

		// create the dialog for the completion of the exercise
		if ( inAnApplet )
			applet_finish_dialog = new AExerciseGUIStats(
					inAnApplet );
		else
			finish_dialog = new ExerciseGUIStats(
					inAnApplet, this, false );
	}


	// run the exercise from a lesson passed in;
	// supplements the normal show() method
	public void show( Lesson lesson ) {

		// salt away the lesson so we can refer to it later
		this.lesson = lesson;

		// set up the labels and text
		title_label.setText( lesson.title );
		descr_label.setText( lesson.descr );
		card_layout.show( text_panel, Integer.toString( lesson.type ));
		if ( lesson.type == Lesson.DRILL ) {
			reference_text_area.setText( lesson.data );
			entry_text_area.setText( "" );
		}
		else {
			paragraph_text_area.setText( lesson.data );
		}

		// start with a clean set of keying statistics
		key_stats.reset();

		// initialize our pointer to the start of the
		// string being used for reference
		progress = 0;
		cursor = 0;
		show();
	}


	// override the show with something that creates the selection
	// cursor after the component is rendered, otherwise we don't
	// get to see it until after the first character press
	public void show() {

		// call the normal show, then select
		super.show();

		// if we have a current lesson select the right part
		// of it
		if ( lesson != null ) {
			if ( lesson.type == Lesson.DRILL )
				reference_text_area.select(
						cursor, cursor + 1 );
			else
				paragraph_text_area.select(
						cursor, cursor + 1 );
		}
	}


	// override dispose to get rid of our stats dialog if in applet
	// mode, before calling the proper dispose method
	public void dispose() {
		if ( inAnApplet )
			applet_finish_dialog.dispose();
		super.dispose();
	}


	// handle dismiss, close, etc by just hiding ourselves, and
	// any dangling stats dialog
	private void handle_menu_close() {
		if ( inAnApplet )
			applet_finish_dialog.hide();
		hide();
	}


	// handle restart of the lesson by re-calling show with our
	// saved lesson - wonderful
	private void handle_menu_restart() {
		show( lesson );
	}


	// return the time in milliseconds between successive calls,
	// for timing key strikes
	private int delta_time() {

		// get the time now, and subtract from it the time we
		// last got a call
		long now_timestamp = System.currentTimeMillis();
		int delta = (int)( now_timestamp - keypress_timestamp );

		// update our note of when we were last called
		keypress_timestamp = now_timestamp;

		// now return the interval in mS
		return ( delta );
	}


	// update all the statistics we have to given a wanted char, and
	// whether it was hit (true) or missed (false)
	private void update_stats( char char_wanted, boolean hit_ok ) {

		// find the time since the last key was pressed
		int delta_t = delta_time();

		// if the time is something we should note, update the
		// statistics - ignore things over a certain threshold
		// to avoid skewing the statistics in the event of a
		// break in mid-exercise
		if ( delta_t > 0 && delta_t < MAX_DELTA ) {

			// first update our lesson-specific statistics
			if ( hit_ok )
				key_stats.struck( delta_t );
			else
				key_stats.missed( delta_t );

			// now update the global key-specific ones
			KeyStatistics k = stats_set.get_statistics(
					char_wanted );
			if ( k != null )
				if ( hit_ok )
					k.struck( delta_t );
				else
					k.missed( delta_t );
		}
	}


	// handle a key press in the frame
	private void handle_key_press( char char_struck ) {

		// ignore if no lesson given to us yet
		if ( lesson != null ) {

			// pass the char to the appropriate handling routine
			if ( lesson.type == Lesson.DRILL )
				handle_key_press_drill( char_struck );
			else
				handle_key_press_paragraph( char_struck );

			// now, if the lesson end was reached update the
			// results dialog with the timings
			if ( progress == lesson.data.length() ) {
				if ( inAnApplet )
					applet_finish_dialog.show(
							lesson, key_stats );
				else
					finish_dialog.show(
							lesson, key_stats );

				// refresh the statistics GUI window if we have
				// been given one
				if ( statistics != null )
					statistics.refresh();
			}
		}
	}
	private void handle_key_press_drill( char char_struck ) {

		// ignore if we are at the end of the reference string
		if ( progress < lesson.data.length() ) {

			// compare the key pressed to the one expected
			char char_wanted = lesson.data.charAt( progress );
			if ( char_struck == char_wanted ) {

				// update the timings and statistics
				update_stats( char_wanted, true );

				// update the input text area
				entry_text_area.appendText(
					lesson.data.substring(
					progress, progress + 1 ));
				// advance the cursors
				progress++; cursor++;
			}
			else {

				// update the timings and statistics
				update_stats( char_wanted, false );

				// update the input text area
				if ( char_wanted == '\n' ) {
					entry_text_area.appendText(
							BADCR + "\n" );
					// advance the cursors
					progress++; cursor++;
				}
				else {
					entry_text_area.appendText(
							BADKEY );
					// advance the cursors
					progress++; cursor++;
				}
			}

			// highlight progress through the exercise
			reference_text_area.select(
					cursor, cursor + 1 );
		}
	}
	private void handle_key_press_paragraph( char char_struck ) {

		// ignore if we are at the end of the reference string
		if ( progress < lesson.data.length() ) {

			// compare the key pressed to the one expected
			char char_wanted = lesson.data.charAt( progress );
			if ( char_struck == char_wanted ) {

				// update the timings and statistics
				update_stats( char_wanted, true );

				// advance the cursors
				progress++; cursor++;
			}
			else {

				// wrong key - handle according to type
				if ( char_struck == '\b' ) {
					if ( progress > 0 && cursor > 0
					&& lesson.data.charAt( progress - 1 )
							!= '\n' ) {
						// back up one character
						progress--; cursor--;
						paragraph_text_area.replaceText(
							lesson.data.substring(
							progress, progress + 1 ),
							cursor, cursor + 1 );
					}
				}
				else {
					// update the timings and statistics
					update_stats( char_wanted, false );

					// update the paragraph typing area
					if ( char_wanted == '\n' ) {
						paragraph_text_area.insertText(
							BADCR, cursor );
						cursor++;
						// advance the cursors
						progress++; cursor++;
					}
					else {
						paragraph_text_area.replaceText(
							BADKEY,
							cursor, cursor + 1 );
						// advance the cursors
						progress++; cursor++;
					}
				}
			}

			// highlight progress through the exercise
			paragraph_text_area.select(
					cursor, cursor + 1 );
		}
	}


	// reset selection in text areas when focus (re)gained;
	// this gives a way to redisplay the expected next character
	private void handle_text_got_focus() {

		// ignore if no lesson given to us yet
		if ( lesson != null ) {
			if ( lesson.type == Lesson.DRILL )
				reference_text_area.select(
						cursor, cursor + 1 );
			else
				paragraph_text_area.select(
						cursor, cursor + 1 );
		}
	}

	// handle GUI component actions and events
	public boolean action( Event evt, Object arg ) {
		if ( evt.target instanceof MenuItem ) {
			if ( evt.target == file_close )
				handle_menu_close();
			else if ( evt.target == file_restart )
				handle_menu_restart();
		}
		return ( false );
	}
	public boolean handleEvent( Event evt ) {
		if ( evt.id == Event.WINDOW_DESTROY ) {
			handle_menu_close();
		}
		return ( super.handleEvent( evt ));
	}
	public boolean keyDown( Event evt, int key ) {
		handle_key_press( (char)key );
		return ( true );
	}
	public boolean gotFocus( Event evt, Object arg ) {
		if ( evt.target instanceof TextArea ) {
			handle_text_got_focus();
		}
		return ( true );
	}


	// register a stats GUI whose refresh we should call when
	// a lesson is completed
	public void register_statistics( StatisticsGUI statistics ) {
		this.statistics = statistics;
	}


	// test code
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		ExerciseGUI exercise = new ExerciseGUI( false, 
			new KBStatistics ( new KBLayout( "US" )));
		exercise.pack();
		exercise.show( new Lesson ( Lesson.DRILL,
			"Test drill title", "Test drill description",
			"Line 1 test text for a drill window\n" +
			"Line 2 test text\n" ));
	}
}
