//
// JTypist v3.0 - Java typing tutor program for UNIX systems
// Copyright (C) 1998  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KBStatistics class - holds the statistics for keyboard usage
//

public class KBStatistics {

	// will keep stats for each key we monitor according to
	// the keyboard layout we are given
	private KBLayout kblayout;		// KB layout we use
	private KeyStatistics stats[];		// stats for keys in layout


	// constructor to create a customized stats set based on
	// the keyboard layout passed in
	public KBStatistics( KBLayout kblayout ) {

		// dimension up the stats array for the number
		// of keys defined in the layout
		this.kblayout = kblayout;
		stats = new KeyStatistics[ kblayout.keymap_table.length ];

		// now initialize the elements of this array
		for ( int index = 0; index < stats.length; index++ )
			stats[ index ] = new KeyStatistics();
	}


	// find the statistics for a key, a finger, and globally
	// (note that only key-based statistics register updates)
	public KeyStatistics get_statistics( char character ) {

		// search the keymap table for this character
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			if ( kblayout.keymap_table[ index ].character
					== character )

				// return the object directly so it
				// can be updated by the caller
				return ( stats[ index ] );
		}
		return ( null );
	}
	public KeyStatistics get_statistics( int finger ) {

		// search the keymap table for keys operated by this
		// finger, and sum the stats in a return object
		KeyStatistics return_stats = null;
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			if ( kblayout.keymap_table[ index ].finger == finger ) {

				// found a matching entry
				if ( return_stats == null )
					return_stats = new KeyStatistics();
				return_stats.add( stats[ index ] );
			}
		}
		return ( return_stats );
	}
	public KeyStatistics get_statistics() {

		// sum up the stats of the complete layout
		KeyStatistics return_stats = new KeyStatistics();
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			return_stats.add( stats[ index ] );
		}
		return ( return_stats );
	}


	// query the KB layout this set is using
	public KBLayout get_kblayout() {
		return ( kblayout );
	}


	// reset the complete statistics set to initial state
	public void reset() {
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			stats[ index ].reset();
		}
	}


	// printout method
	public String toString() {
		String ret_string = "KBStatistics{stats@";
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			ret_string += "[character="
				+ kblayout.keymap_table[ index ].character + "]"
				+ stats[ index ];
		}
		return ( ret_string + "}" );
	}


	// test code
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		KBLayout k = new KBLayout( "US" );
		KBStatistics ks = new KBStatistics( k );
		for ( int i = 0; i < k.keymap_table.length; i++ ) {
			char key = k.keymap_table[ i ].character;
			for ( int j = 0; j < key; j++ )
				ks.get_statistics( key ).struck( j % 100 );
			for ( int j = 0; j < key / 10; j++ )
				ks.get_statistics( key ).missed( j % 100 );
		}

		System.out.println( ks );
		for ( int i = 0; i < k.keymap_table.length; i++ ) {
			char key = k.keymap_table[ i ].character;
			System.out.println( "Key " + key + " : " +
					ks.get_statistics( key ));
		}

		if ( ks.get_statistics( KBLayout.LH_THUMB ) != null )
			System.out.println( "Finger " +
				KBLayout.LH_THUMB + " : "
				+ ks.get_statistics( KBLayout.LH_THUMB ));
		if ( ks.get_statistics( KBLayout.LH_INDEX ) != null )
			System.out.println( "Finger " +
				KBLayout.LH_INDEX + " : "
				+ ks.get_statistics( KBLayout.LH_INDEX ));
		if ( ks.get_statistics( KBLayout.LH_MIDDLE ) != null )
			System.out.println( "Finger " +
				KBLayout.LH_MIDDLE + " : "
				+ ks.get_statistics( KBLayout.LH_MIDDLE ));
		if ( ks.get_statistics( KBLayout.LH_RING ) != null )
			System.out.println( "Finger " +
				KBLayout.LH_RING + " : "
				+ ks.get_statistics( KBLayout.LH_RING ));
		if ( ks.get_statistics( KBLayout.LH_PINKY ) != null )
			System.out.println( "Finger " +
				KBLayout.LH_PINKY + " : "
				+ ks.get_statistics( KBLayout.LH_PINKY ));
		if ( ks.get_statistics( KBLayout.RH_THUMB ) != null )
			System.out.println( "Finger " +
				KBLayout.RH_THUMB + " : "
				+ ks.get_statistics( KBLayout.RH_THUMB ));
		if ( ks.get_statistics( KBLayout.RH_INDEX ) != null )
			System.out.println( "Finger " +
				KBLayout.RH_INDEX + " : "
				+ ks.get_statistics( KBLayout.RH_INDEX ));
		if ( ks.get_statistics( KBLayout.RH_MIDDLE ) != null )
			System.out.println( "Finger " +
				KBLayout.RH_MIDDLE + " : "
				+ ks.get_statistics( KBLayout.RH_MIDDLE ));
		if ( ks.get_statistics( KBLayout.RH_RING ) != null )
			System.out.println( "Finger " +
				KBLayout.RH_RING + " : "
				+ ks.get_statistics( KBLayout.RH_RING ));
		if ( ks.get_statistics( KBLayout.RH_PINKY ) != null )
			System.out.println( "Finger " +
				KBLayout.RH_PINKY + " : "
				+ ks.get_statistics( KBLayout.RH_PINKY ));
		System.out.println( "Global : "
					+ ks.get_statistics());
	}
}
