//
// JTypist v3.0 - Java typing tutor program for UNIX systems
// Copyright (C) 1998  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KeyStatistics class - counts hits, misses, timings, and calculates
// WPM values when requested
//

public class KeyStatistics {

	// statistics for the count of times a key is hit, how
	// long it takes to hit, and how many were missed
	private int struck_count = 0;
	private int missed_count = 0;
	private int ms_expended	= 0;


	// register a successful hit on a key
	public void struck( int msec ) {
		struck_count++;
		ms_expended += msec;
	}


	// register a miss on a key
	public void missed( int msec ) {
		missed_count++;
		ms_expended += msec;
	}


	// reset the statistics set to the initial state
	public void reset() {
		struck_count = 0;
		missed_count = 0;
		ms_expended = 0;
	}


	// return the hit count of a key, miss count, and time expended
	public int get_strikes() { return ( struck_count ); }
	public int get_misses() { return ( missed_count ); }
	public int get_ms_expended() { return ( ms_expended ); }


	// calculate % accuracy and errors on this key
	public int get_pc_accuracy() {
		if ( struck_count + missed_count > 0 )
			return (( struck_count * 100 )
					/ ( struck_count + missed_count ));
		else
			return ( 0 );
	}
	public int get_pc_errors() {
		if ( struck_count + missed_count > 0 )
			return ( 100 - get_pc_accuracy() );
		else
			return ( 0 );
	}


	// calculate the WPM from hits and timings; adjust for errors
	// if requested
	public int get_WPM( boolean adjusted ) {

		// if any time at all registered OK to continue
		if ( ms_expended > 0 ) {

			// calculate effective elapsed time
			float secs = (float)ms_expended / 1000.0f;
			float mins = secs / 60.0f;

			// calculate effective words (5 chars) - here we
			// include missed keys, since even though they were
			// missed something was struck; we'll account for
			// missed in adjusted WPM later in a large way
			float effective_words = (float)( struck_count
						+ missed_count) / 5.0f;

			// find WPM, and take it as an integer
			int wpm = (int)( effective_words / mins );

			// apply a word penalty for each miss if adjusting,
			// and clamp -ve results to zero (can't really get
			// any lower)
			if ( adjusted ) {
				wpm -= missed_count;
				if ( wpm < 0 )
					wpm = 0;
			}
			return ( wpm );
		}
		else
			// no timings taken
			return ( 0 );
	}


	// method to combine this stats set with another
	public void add( KeyStatistics kbstats ) {
		this.struck_count += kbstats.get_strikes();
		this.missed_count += kbstats.get_misses();
		this.ms_expended += kbstats.get_ms_expended();
	}


	// printout method
	public String toString() {
		return ( "KeyStatistics{struck_count=" + get_strikes()
				+ ",missed_count=" + get_misses()
				+ ",ms_expended=" + get_ms_expended()
				+ ",get_pc_accuracy()=" + get_pc_accuracy()
				+ ",get_pc_errors()=" + get_pc_errors()
				+ ",get_WPM(false)=" + get_WPM( false )
				+ ",get_WPM(true)=" + get_WPM( true ) + "}" );
	}


	// test code
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		KeyStatistics k = new KeyStatistics();
		System.out.println( "initial : " + k );
		k.struck( 200 );
		System.out.println( "1@200mS : " + k );
		KeyStatistics k2 = new KeyStatistics();
		k2.struck( 300 ); k2.missed( 100 );
		k.add( k2 );
		System.out.println( "+1@300mS/1@100mS : " + k );

		for ( int i = 0; i < 1000; i++ )
			k.struck( i );
		for ( int i = 0; i < 100; i++ )
			k.missed( i );
		System.out.println( "full : " + k );
	}
}
