//
// JTypist v3.0 - Java typing tutor program for UNIX systems
// Copyright (C) 1998  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// StatisticsGUI class - creates a frame to display typing statistics
//
import java.awt.*;
import java.applet.*;
import java.util.Vector;

public class StatisticsGUI extends Frame {
	private boolean inAnApplet = true;

	// menubar GUI components
	private MenuBar menu_bar;
	private Menu file_menu;
	private MenuItem file_reset, file_close;
	private Menu view_menu;
	private CheckboxMenuItem view_adjWPM;
	private CheckboxMenuItem view_finger;
	private CheckboxMenuItem view_alpha;
	private CheckboxMenuItem view_shiftedalpha;
	private CheckboxMenuItem view_numeric;
	private CheckboxMenuItem view_basic;
	private CheckboxMenuItem view_figures;

	// GUI components
	private Panel card_panel;
	private CardLayout card_layout;
	private Button refresh_button;
	private Vector acc_gauges = new Vector();
	private Vector wpm_gauges = new Vector();
	private Vector stats_obj = new Vector();

	// somewhat arbitrary limits for warnings/errors on gauges; this
	// does the coloured gauges to try to help pick out the keys that
	// need more practice
	private final int ERROR_ACCURACY = 70;		// <70% is bad
	private final int WARN_ACCURACY = 90;		// <90% is a concern
	private final int ERROR_SPEED = 10;		// <10WPM is bad
	private final int WARN_SPEED = 30;		// <30WPM is a concern

	// statistics set we are working with, for reference
	KBStatistics stats_set = null;


	// constructor initializes the whole GUI set
	public StatisticsGUI( boolean inAnApplet, KBStatistics stats_set ) {
		this.inAnApplet = inAnApplet;

		// create GUI components
		setTitle( "Typist v3.0 Typing Statistics Window" );
		setLayout( new BorderLayout() );

		// note the stats set for reference
		this.stats_set = stats_set;

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu( "File" );
		menu_bar.add( file_menu );
		file_reset = new MenuItem( "Reset All" );
		file_menu.add( file_reset );
		file_close = new MenuItem( "Close Window" );
		file_menu.add( file_close );

		// first part of the view menu
		view_menu = new Menu( "View" );
		menu_bar.add( view_menu );
		view_adjWPM = new CheckboxMenuItem( "Adjusted WPM" );
		view_menu.add( view_adjWPM );
		view_menu.add( new MenuItem( "-" ));

		// create the refresh button
		Panel toolbar = new Panel();
		toolbar.setLayout( new BorderLayout() );
		add( "North", toolbar );

		Panel controls = new Panel();
		controls.setLayout( new FlowLayout() );
		toolbar.add( "West", controls );
		refresh_button = new Button( "Refresh" );
		controls.add( refresh_button );
		Panel kbtype = new Panel();
		kbtype.setLayout( new FlowLayout() );
		toolbar.add( "East", kbtype );
		Label kbtype_label = new Label( "Keyboard type: "
				+ stats_set.get_kblayout().kb_id );
		kbtype.add( kbtype_label );

		// create a card panel for the stats to live on
		card_panel = new Panel();
		card_layout = new CardLayout();
		card_panel.setLayout( card_layout );

		// find the maximum number of statistics any panel
		// in the set could have (excluding the finger panel,
		// which we conveniently assume cannot be the largest -
		// there are probably over 100 stats in five groups);
		// this is used for padding the panels with null labels
		// so the sizes of all the gauges and labels are equal
		int max_num_stats =
			Math.max( size_stats_panel( stats_set,
					KBLayout.UNSHIFTED_ALPHA ),
			Math.max( size_stats_panel( stats_set,
					KBLayout.SHIFTED_ALPHA ),
			Math.max( size_stats_panel( stats_set,
					KBLayout.NUMERIC ),
			Math.max( size_stats_panel( stats_set,
					KBLayout.BASIC ),
				size_stats_panel( stats_set,
					KBLayout.FIGURES ) ))));

		// create the panel of finger statistics
		Panel fstats_panel = make_finger_stats_panel( stats_set,
				max_num_stats );
		fstats_panel.setBackground( Color.white );
		view_finger = new CheckboxMenuItem( "Finger Statistics" );
		view_menu.add( view_finger );
		card_panel.add( view_finger.getLabel(), fstats_panel );

		// create a panel for unshifted character stats, shifted
		// ones, and all of the other stuff, and complete the
		// view menu for populated panels
		Panel ustats_panel = make_stats_panel( stats_set,
				KBLayout.UNSHIFTED_ALPHA, max_num_stats );
		if ( ustats_panel != null ) {
			ustats_panel.setBackground( Color.white );
			view_alpha = new CheckboxMenuItem( "Alphabetic Keys" );
			view_menu.add( view_alpha );
			card_panel.add( view_alpha.getLabel(), ustats_panel );
		}
		Panel sstats_panel = make_stats_panel( stats_set,
				KBLayout.SHIFTED_ALPHA, max_num_stats );
		if ( sstats_panel != null ) {
			sstats_panel.setBackground( Color.white );
			view_shiftedalpha = new CheckboxMenuItem( "Shifted Alphabetic Keys" );
			view_menu.add( view_shiftedalpha );
			card_panel.add( view_shiftedalpha.getLabel(), sstats_panel );
		}
		Panel nstats_panel = make_stats_panel( stats_set,
				KBLayout.NUMERIC, max_num_stats );
		if ( nstats_panel != null ) {
			nstats_panel.setBackground( Color.white );
			view_numeric = new CheckboxMenuItem( "Numeric Keys" );
			view_menu.add( view_numeric );
			card_panel.add( view_numeric.getLabel(), nstats_panel );
		}
		Panel bstats_panel = make_stats_panel( stats_set,
				KBLayout.BASIC, max_num_stats );
		if ( bstats_panel != null ) {
			bstats_panel.setBackground( Color.white );
			view_basic = new CheckboxMenuItem( "Other Keys" );
			view_menu.add( view_basic );
			card_panel.add( view_basic.getLabel(), bstats_panel );
		}
		Panel ostats_panel = make_stats_panel( stats_set,
				KBLayout.FIGURES, max_num_stats );
		if ( ostats_panel != null ) {
			ostats_panel.setBackground( Color.white );
			view_figures = new CheckboxMenuItem( "Figures Keys" );
			view_menu.add( view_figures );
			card_panel.add( view_figures.getLabel(), ostats_panel );
		}

		// add the card panel to the main panel
		view_finger.setState( true );
		card_layout.show( card_panel, view_finger.getLabel() );
		add( "Center", card_panel );
	}


	// create a panel containing finger statistics
	private Panel make_finger_stats_panel( KBStatistics stats_set,
			int pad_stats ) {

		// start with a set of column headings
		Panel panel = new Panel();
		panel.setLayout( new GridLayout( 0, 9 ));
		for ( int index = 0; index < 3; index++ ) {
			panel.add( new Label() );
			panel.add( new Label( "Accuracy", Label.CENTER ));
			panel.add( new Label( "Speed", Label.CENTER ));
		}

		// add data labels for all the fingers we could monitor;
		// note that if a finger actually does no keys then it
		// will not appear on the stats panel.  For the data
		// source for refreshes we note the finger ID as an
		// integer object
		int num_stats = 0;
		if ( stats_set.get_statistics( KBLayout.LH_THUMB ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.LH_THUMB ),
				"LH Thumb" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.LH_INDEX ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.LH_INDEX ),
				"LH Index" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.LH_MIDDLE ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.LH_MIDDLE ),
				"LH Middle" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.LH_RING ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.LH_RING ),
				"LH Ring" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.LH_PINKY ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.LH_PINKY ),
				"LH Little" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.RH_THUMB ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.RH_THUMB ),
				"RH Thumb" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.RH_INDEX ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.RH_INDEX ),
				"RH Index" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.RH_MIDDLE ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.RH_MIDDLE ),
				"RH Middle" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.RH_RING ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.RH_RING ),
				"RH Ring" );
			num_stats++;
		}
		if ( stats_set.get_statistics( KBLayout.RH_PINKY ) != null ) {
			make_disp_entry( panel, new Integer( KBLayout.RH_PINKY ),
				"RH Little" );
			num_stats++;
		}

		// pad the panel to the maximum size, rounded up to the
		// next threesome
		for ( ; num_stats < ( ( pad_stats / 3 ) * 3 ) + 3;
				num_stats++ )
			make_disp_entry( panel, null, null );

		// return the panel - note how this one can't be null since
		// there must be some statistics on it!
		return ( panel );
	}


	// count how many statistics we would display on a panel, given
	// this set and grouping - used to search for largest group
	private int size_stats_panel( KBStatistics stats_set,
			int char_group ) {

		// find the KB layout for the stats
		KBLayout kblayout = stats_set.get_kblayout();

		// now count the statistics for this group
		int num_stats = 0;
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {
			if ( kblayout.keymap_table[ index ].char_group
					== char_group ) {

				num_stats++;
			}
		}

		// return the count
		return ( num_stats );
	}


	// create a complete panel of labels/gauges given a char group, keymap
	// table, and some luck
	private Panel make_stats_panel( KBStatistics stats_set,
			int char_group, int pad_stats ) {

		// create the new panel and add headings
		Panel panel = new Panel();
		panel.setLayout( new GridLayout( 0, 9 ));
		for ( int index = 0; index < 3; index++ ) {
			panel.add( new Label() );
			panel.add( new Label( "Accuracy", Label.CENTER ));
			panel.add( new Label( "Speed", Label.CENTER ));
		}

		// find the KB layout for the stats
		KBLayout kblayout = stats_set.get_kblayout();

		// now add the data gauges
		int num_stats = 0;
		for ( int index = 0; index < kblayout.keymap_table.length;
				index++ ) {

			// if this char is in our group then process it
			if ( kblayout.keymap_table[ index ].char_group
					== char_group ) {

				// create the stats entry for this key, giving
				// the stats object directly as the data
				// source for refreshes
				char character =
					kblayout.keymap_table[ index ].character;

				// translate \n into Enter, and ' ' to space
				String legend =
					( character == '\n' ) ?
					"Enter" :
						( character == ' ' ) ?
						"Space" :
							"Key '"
					+ new Character( character ).toString()
							+ "'";

				// create the entry, unless for some reason
				// we aren't keeping stats on this character
				if ( stats_set.get_statistics( character )
						!= null ) {
					make_disp_entry( panel,
					stats_set.get_statistics( character ),
						legend );
					num_stats++;
				}
			}
		}

		// return null if nothing on the panel, otherwise pad the
		// panel to the largest size and return it
		if ( num_stats > 0 ) {

			// pad the panel to the maximum size, rounded up to the
			// next threesome
			for ( ; num_stats < ( ( pad_stats / 3 ) * 3 ) + 3;
					num_stats++ )
				make_disp_entry( panel, null, null );

			// return the panel
			return ( panel );
		}
		else
			// nothing on this one - return null
			return ( null );
	}


	// create a trio of components showing the specified stats
	// set and add the trio to the panel given
	private void make_disp_entry( Panel panel, Object stats_object,
			String legend ) {

		// if there is a source data object add the gauges
		if ( stats_object != null ) {

			// add a legend label and two readout gauges
			panel.add( new Label( legend, Label.RIGHT ));
			Gauge g = new Gauge( 0, 100, 0, "%" );
			g.setBackground( Color.lightGray );
			acc_gauges.addElement( g ); panel.add( g );
			g = new Gauge( 0, 100, 0, " WPM" );
			g.setBackground( Color.lightGray );
			wpm_gauges.addElement( g ); panel.add( g );
			stats_obj.addElement( stats_object );
		}
		else {
			// add a null entry for padding - three blank
			// labels
			panel.add( new Label() );
			panel.add( new Label() );
			panel.add( new Label() );
		}
	}


	// refresh the labels with the latest stats data
	public void refresh() {

		// run through the list of labels we need to update
		for ( int index = 0; index < stats_obj.size(); index++ ) {

			// note the gauges we are going to update the
			// data into
			Gauge acc =
				(Gauge)acc_gauges.elementAt( index );
			Gauge wpm =
				(Gauge)wpm_gauges.elementAt( index );

			// if the data source object is an integer then
			// refetch the finger stats, since they are
			// computed on demand; otherwise use the stats
			// data source directly
			KeyStatistics key_stats = null;
			Object stats_object = stats_obj.elementAt( index );
			if ( stats_object instanceof Integer ) {
				key_stats = stats_set.get_statistics(
					(int)((Integer)stats_object).intValue() );
			}
			else if ( stats_object instanceof KeyStatistics ) {
				key_stats = (KeyStatistics)stats_object;
			}

			// if no time expended then this key or set of
			// keys has never been struck
			if ( key_stats.get_ms_expended() > 0 ) {

				// get the statistics values
				int accuracy = key_stats.get_pc_accuracy();
				int speed = key_stats.get_WPM(
						view_adjWPM.getState() );

				// colour the gauges to warn of underperformance
				if ( accuracy <= ERROR_ACCURACY )
					acc.setForeground( Color.red );
				else if ( accuracy <= WARN_ACCURACY )
					acc.setForeground( Color.orange );
				else
					acc.setForeground( Color.black );
				if ( speed <= ERROR_SPEED )
					wpm.setForeground( Color.red );
				else if ( speed <= WARN_SPEED )
					wpm.setForeground( Color.orange );
				else
					wpm.setForeground( Color.black );

				// update the gauges with current values
				acc.set_value( key_stats.get_pc_accuracy() );
				wpm.set_value( key_stats.get_WPM(
						view_adjWPM.getState() ));
			}
			else {

				// reset the gauges to nothing
				acc.setForeground( Color.black );
				wpm.setForeground( Color.black );
				acc.set_value( acc.get_min() );
				wpm.set_value( wpm.get_min() );
			}
		}
	}


	// override show with something to refresh the stats
	public void show() {

		// refresh the complete stats set before displaying
		refresh();
		super.show();
	}

	// handle dismiss, close, etc by just hiding ourselves
	private void handle_close_button() {

		// simply just hide this frame from view
		hide();
	}


	// handle reset request by resetting the underlying stats
	// then redisplaying
	private void handle_reset_button() {
		stats_set.reset();
		refresh();
	}


	// handle view change by displaying the card selected
	private void handle_view_change( CheckboxMenuItem cb_menuitem ) {

		// emulate the groups thing by deselecting all
		// except the pushed one; can't find a better way
		// though I suspect there might be one
		view_finger.setState( false );
		view_alpha.setState( false );
		view_shiftedalpha.setState( false );
		view_numeric.setState( false );
		view_basic.setState( false );
		view_figures.setState( false );
		cb_menuitem.setState( true );

		// display the right card panel, using the label string
		// as the key to the card
		card_layout.show( card_panel, cb_menuitem.getLabel() );
		refresh();
	}

	// handle GUI component actions and events
	public boolean action( Event evt, Object arg ) {
		if ( evt.target instanceof Button ) {
			if ( evt.target == refresh_button ) {
				refresh();
			}
		}
		else if ( evt.target instanceof MenuItem ) {
			if ( evt.target == file_close ) {
				handle_close_button();
			}
			else if ( evt.target == file_reset ) {
				handle_reset_button();
			}
		}
		return ( false );
	}
	public boolean handleEvent( Event evt ) {
		if ( evt.id == Event.WINDOW_DESTROY ) {
			handle_close_button();
		}
		else if ( evt.target instanceof CheckboxMenuItem ) {
			if ( evt.target == view_finger ||
					evt.target == view_alpha ||
					evt.target == view_shiftedalpha ||
					evt.target == view_numeric ||
					evt.target == view_basic ||
					evt.target == view_figures )
				handle_view_change(
					(CheckboxMenuItem)evt.target );
			else if ( evt.target == view_adjWPM )
				refresh();
		}
		return ( super.handleEvent( evt ));
	}


	// test code
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		KBStatistics kk = new KBStatistics(
					new KBLayout( "US" ));
		kk.get_statistics( ' ' ).struck( 200 );
		StatisticsGUI stats = new StatisticsGUI( false, kk );
		stats.pack(); stats.show();
		kk.get_statistics( 'q' ).struck( 200 );
		kk.get_statistics( 'q' ).struck( 250 );
		kk.get_statistics( 'q' ).missed( 50 );
	}
}
